###########################################################################
#
# win32 makefile -- src/oaiservr
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the
# University of Waikato, New Zealand.
#
# Copyright (C) 1999  The New Zealand Digital Library Project
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################

GSDLHOME = ..\..
GSDL_VC4 = 0
DLL = 0
DLLDEBUG = 0
ENABLE_ACCENTFOLD = 1

STLPATH = "$(GSDLHOME)\packages\windows\stlport\stlport"

recptDir = $(GSDLHOME)\src\recpt
colservrDir = $(GSDLHOME)\src\colservr
srcDir = $(GSDLHOME)\src

AR = lib
CC = cl

!IF $(GSDL_VC4)
CPPFLAGS = -GX
!ELSE
CPPFLAGS = -EHsc
!ENDIF

!IF $(DLL)
CPPFLAGS = $(CPPFLAGS) -MD
!ELSE
!IF ($(DLLDEBUG))
CPPFLAGS = $(CPPFLAGS) -MDd
!ENDIF
!ENDIF

LDFLAGS =
DEFS = -D__WIN32__ -DHAVE_CONFIG_H -DPARADOCNUM -D_LITTLE_ENDIAN -DSHORT_SUFFIX -D_CRT_SECURE_NO_DEPRECATE
INCLUDES = -I"$(GSDLHOME)\src\recpt" -I"$(GSDLHOME)\lib" -I"$(GSDLHOME)\src\colservr" \
	   -I"$(GSDLHOME)\packages\mg" -I"$(GSDLHOME)\packages\mg\lib" \
	   -I"$(GSDLHOME)\packages\mg\src\text" -I"$(GSDLHOME)\packages\windows\gdbm\gdbm" \
	   -I"$(GSDLHOME)\packages\windows\crypt\crypt" -I"$(GSDLHOME)\src\mgpp\lib" \
	   -I"$(GSDLHOME)\src\mgpp\text" -I"$(GSDLHOME)"
!IF $(GSDL_VC4)
DEFS = $(DEFS) -DGSDL_NAMESPACE_BROKEN -DGSDL_USE_IOS_H -D__STL_NO_NEW_IOSTREAMS
INCLUDES = $(INCLUDES) -I$(STLPATH)
!ENDIF

!IF $(ENABLE_ACCENTFOLD)
AFLIBS = "$(GSDLHOME)\packages\unac\libunac.lib" "$(GSDLHOME)\packages\unac\iconv.lib"
DEFS = $(DEFS) -DENABLE_ACCENTFOLD
!ELSE
AFLIBS = 
!ENDIF

LIBS = "$(GSDLHOME)\packages\windows\gdbm\gdbm\gdbm.lib" \
       "$(GSDLHOME)\packages\windows\crypt\crypt\crypt.lib" \
       "$(GSDLHOME)\src\colservr\gsdlcolservr.lib" \
       "$(GSDLHOME)\lib\gsdllib.lib" \
       "$(GSDLHOME)\packages\mg\src\text\libtextin.lib" \
       "$(GSDLHOME)\packages\mg\lib\libmg.lib" \
       "$(GSDLHOME)\src\mgpp\text\libtextin.lib" \
       "$(GSDLHOME)\src\mgpp\lib\libmg.lib" \
       "$(GSDLHOME)\packages\windows\expat\expat\libexpat.lib" \
	$(AFLIBS)


COMPILE = $(CC) -nologo -c $(CPPFLAGS) $(DEFS) $(INCLUDES)
LINK = $(CC) -nologo $(LDFLAGS) -Fe"$@"

.SUFFIXES:
.SUFFIXES: .c .obj
.c.obj:
	$(COMPILE) $<

.SUFFIXES: .cpp .obj
.cpp.obj:
	$(COMPILE) $< 


COMMONHEADERS = oaiargs.h oaiaction.h oaidispatcher.h oaiconfig.h identityaction.h recordaction.h \
                listrecsaction.h listidsaction.h listsetsaction.h metaformatsaction.h oaitools.h \
                abstractlistaction.h metaformat.h dublincore.h rfc1807.h resumptiontoken.h

HEADERS = $(COMMONHEADERS)


COMMONSOURCES = oaiargs.cpp oaiaction.cpp oaidispatcher.cpp oaiconfig.cpp identityaction.cpp recordaction.cpp \
                abstractlistaction.cpp listrecsaction.cpp listidsaction.cpp listsetsaction.cpp \
                metaformatsaction.cpp oaitools.cpp metaformat.cpp dublincore.cpp rfc1807.cpp \
                resumptiontoken.cpp

OAISOURCES = 

SOURCES = $(COMMONSOURCES) $(OAISOURCES)

RECPTOBJECTS = "$(recptDir)\comtypes.obj" "$(recptDir)\OIDtools.obj" "$(recptDir)\nullproto.obj" "$(recptDir)\recptproto.obj" \
               "$(recptDir)\htmlutils.obj" "$(recptDir)\infodbclass.obj" "$(recptDir)\recptconfig.obj" "$(recptDir)\cgiargs.obj" \
               "$(recptDir)\cgiutils.obj"

COMMONOBJECTS = oaiargs.obj oaiaction.obj oaiconfig.obj oaidispatcher.obj identityaction.obj recordaction.obj \
                abstractlistaction.obj listrecsaction.obj listidsaction.obj listsetsaction.obj \
                metaformatsaction.obj oaitools.obj \
                metaformat.obj dublincore.obj rfc1807.obj resumptiontoken.obj $(RECPTOBJECTS)

OAIOBJECTS = oaimain.obj

OBJECTS = $(COMMONOBJECTS) $(OAIOBJECTS)

OAI_OBJS = $(COMMONOBJECTS) $(OAIOBJECTS) $(LIBS)

EXEC = oaiserver.exe

all: $(EXEC)

clean:
	del *.obj
	del oaiserver.exe
install:
	copy oaiserver.exe "$(GSDLHOME)\cgi-bin"

oaiserver.exe: $(OAI_OBJS)
	     $(LINK) $(OAI_OBJS)
